#pragma once
#include "Game.h"


// The base class for all "units" that take up a position on the game grid.
class Entity : public Renderer {

public:
	// public vars:
	Team team;
	int localOrderTimeLeft;

	// Our orders (could be some kind of queue eventually)
	Vec2 globalOrder;
	Vec2 localOrder;
	LocalOrderType lorderType;

	// current target
	Entity *target;

	// Constructor / Destructor
	Entity();
	~Entity() { };

	// Functions
	void TakeDamage(Entity *from, float amount);


	virtual RenderType GetType() { return RT_ENTITY; }
	virtual void Render() = 0;
	virtual void Destroy();
	virtual void Update();

protected:
	float hp;
	float power;
	float dmg;
	float range;
	int speed;

	int moveWait;
	EntityState state;

	
	void ApplyLocalOrder(Vec2 pos, LocalOrderType orderType, int timeLeft);
	Vec2 GetBestTerrainInRange();
	bool TargetInRange(int x, int y);
	bool TargetInRange();

	virtual void ApplyState(EntityState state);
	bool CanMove(Vec2 pos);
	void MoveTo(Direction dir);
	void SetPos(Vec2 pos);
	Vec2 GetClosestAdj(Vec2 pos);
	Entity *GetNearestEnemy(int dist);
	void DoSplashDmg(Vec2 pos, int dmg, int radius, Team immune);

	virtual void Act() = 0;
	virtual void Attack() = 0;
	void ActAsSoldier();
	void ActAsTurret();

private:

};

// Bases
class Base : public Entity {
public:
	Base(int x, int y, float hp, Team team);
	void Render();
	void Update();
	RenderType GetType(){ return RT_BASE; }

protected:
	void Act();
	void Attack();
};

// Tanks
class Tank : public Entity {
public:
	Tank();
	Tank(int x, int y, float hp, int speed, int power, Color col, Team team);

	void Render();

protected:
	void Act();
	void Attack();

private:
	Vec2 weaponTarget;
	int loadingTime;
};

// Soldier and sniper
class Soldier : public Entity {
public:
	Soldier(int x, int y, SoldierType type, Team team);
	Soldier(int x, int y, float hp, int speed, int power, Color col, Team team);
	void Render();

protected:
	void Act();
	void Attack();

private:
	bool laseron;
	SoldierType type;
};