#pragma once
#include <math.h>

// Our 2d vector class. If we want to add cross-product and dot product etc, it may
// warrant becoming a class. Includes some handy functions, such as basic operators
// and normalization, etc.
struct Vec2 {
	float x;
	float y;
	Vec2() {
		x = -1;
		y = -1;
	}
	Vec2(float nx, float ny) {
		x = nx;
		y = ny;
	}
	bool Valid() { return (x >= 0 && y >= 0); }

	bool operator==( Vec2 &other ){
		return (other.x == x && other.y == y);
	};
	bool operator!=( Vec2 &other ){
		return (other.x != x || other.y != y);
	};
	Vec2 operator+( Vec2 &other ){
		return Vec2(x + other.x, y+ other.y);
	};
	Vec2 operator-( Vec2 &other ){
		return Vec2(x - other.x, y- other.y);
	};
	Vec2 operator*( float factor ){
		return Vec2(x * factor, y * factor);
	};
	float distSqr(Vec2 &other) {
		return (other.x - x) * (other.x - x) + (other.y - y) * (other.y - y);
	}
	float magnitude() {
		return sqrt((float)(x*x + y*y));
	}
	Vec2 Normalize() {
		float mag = magnitude();
		return Vec2(x / mag, y / mag);
	}
};